# !/usr/bin/python
# coding=utf-8
# try:
#     import pymel.core as pm
# except ImportError as error:
#     print(__file__, error)
# from this package:
from mayatk.core_utils import CoreUtils, Preview
from mayatk.edit_utils import EditUtils


class Mirror:
    @staticmethod
    @CoreUtils.undo
    def perform_mirror(*args, **kwargs):
        """Mirror geometry across a given axis.

        Parameters:
            objects (obj): The objects to mirror.
            axis (string): The axis in which to perform the mirror along. case insensitive. (valid: 'x', '-x', 'y', '-y', 'z', '-z')
            axis_pivot (int): The pivot on which to mirror on. valid: 0) Bounding Box, 1) Object, 2) World.
            cut_mesh (bool): Perform a delete along specified axis before mirror.
            merge_mode (int): 0) Do not merge border edges. 1) Border edges merged. 2) Border edges extruded and connected.
            merge_threshold (float): Merge vertex distance.
            delete_original (bool): Delete the original objects after mirroring.
            deleteHistory (bool): Delete non-deformer history on the object(s) before performing the operation.
            uninstance (bool): Un-instance the object(s) before performing the operation.

        Returns:
            (obj) The polyMirrorFace history node if a single object, else None.
        """
        EditUtils.mirror(*args, **kwargs)


class MirrorSlots:
    def __init__(self):
        # Initialize the switchboard and UI here
        self.sb = self.switchboard()
        self.ui = self.sb.mirror
        self.preview = Preview(
            self, self.ui.chk000, self.ui.b000, message_func=self.sb.message_box
        )

        # Connect sliders and checkboxes to preview refresh function
        self.sb.connect_multi(self.ui, "chk001-10", "clicked", self.preview.refresh)

    def perform_operation(self, objects):
        # Read values from UI and execute mirror operation
        axis = self.sb.get_axis_from_checkboxes("chk001-4", self.ui)
        axis_pivot = 2 if self.ui.chk008.isChecked() else 1
        cut_mesh = self.ui.chk005.isChecked()
        merge = self.ui.chk007.isChecked()
        delete_original = self.ui.chk010.isChecked()
        delete_history = self.ui.chk006.isChecked()
        uninstance = self.ui.chk009.isChecked()

        Mirror.perform_mirror(
            objects,
            axis=axis,
            axis_pivot=axis_pivot,
            cut_mesh=cut_mesh,
            merge_mode=2 if merge else 0,
            delete_original=delete_original,
            delete_history=delete_history,
            uninstance=uninstance,
        )


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    import os
    from uitk import Switchboard

    parent = CoreUtils.get_main_window()
    ui_file = os.path.join(os.path.dirname(__file__), "mirror.ui")
    sb = Switchboard(parent, ui_location=ui_file, slot_location=MirrorSlots)

    sb.current_ui.set_attributes(WA_TranslucentBackground=True)
    sb.current_ui.set_flags(
        Tool=True, FramelessWindowHint=True, WindowStaysOnTopHint=True
    )
    sb.current_ui.set_style(theme="dark", style_class="translucentBgWithBorder")

    sb.current_ui.show(pos="screen", app_exec=True)

# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------



# !/usr/bin/python
# coding=utf-8
try:
    import pymel.core as pm
except ImportError as error:
    print(__file__, error)
# from this package:
from mayatk import core_utils


class DisplayUtils:
    @staticmethod
    @core_utils.CoreUtils.undo
    def set_visibility(
        elements, visibility=True, include_ancestors=True, affect_layers=True
    ):
        """Sets the visibility of specified elements in the Maya scene.
        It accepts a wide variety of inputs for the elements parameter, including strings,
        PyMEL objects, or lists of objects. It can also optionally affect the visibility of layers
        and ancestor nodes.

        Parameters:
        elements (str | pm.nt.DependNode | list): A string that represents a Maya object type,
                       a wildcard expression, a single PyMEL object, or a list of PyMEL objects.
        visibility (bool): The visibility state to apply. If True, elements are shown; if False, elements are hidden.
        include_ancestors (bool): If True, will also set visibility for all ancestor transform nodes of the elements.
        affect_layers (bool): If True, will ensure that all layers except the default layer have their visibility set.

        Usage:
        set_visibility('geometry', visibility=True)  # Shows all geometry and their ancestors, affects layers.
        set_visibility('lights', visibility=False, include_ancestors=False)  # Hides all lights without affecting their ancestors.
        set_visibility('nurbsCurves', visibility=True, affect_layers=False)  # Shows all nurbsCurves, doesn't affect layers.
        set_visibility([my_geo1, my_geo2], visibility=False)  # Hides specific geometries provided in a list.
        set_visibility('pCube*', visibility=True)  # Shows all objects with names starting with 'pCube'.
        """
        if affect_layers:
            # Set visibility for all layers except the default layer
            for layer in pm.ls(type="displayLayer"):
                if layer.name() != "defaultLayer" and not layer.isReferenced():
                    try:
                        layer.visibility.set(visibility)
                    except pm.MayaAttributeError:
                        pass  # Skip the layer if visibility cannot be set

        common_visibility_types = [
            "mesh",
            "nurbsCurve",
            "nurbsSurface",
            "subdiv",
            "camera",
            "joint",
            "light",
            "locator",
            "transform",
        ]
        elements = [elements] if isinstance(elements, str) else elements
        if set(elements).intersection(common_visibility_types):
            scene_elements = pm.ls(type=elements)
        else:
            # Use it as a search pattern or type name
            scene_elements = pm.ls(elements)

        for element in scene_elements:
            if include_ancestors:
                # Set visibility for ancestor transform nodes
                ancestors = [
                    ancestor
                    for ancestor in element.getAllParents()
                    if isinstance(ancestor, pm.nt.Transform)
                ]
                for ancestor in ancestors:
                    try:
                        ancestor.visibility.set(visibility)
                    except pm.MayaAttributeError:
                        pass  # Skip the ancestor if visibility cannot be set

            # Set the visibility of the element
            try:
                element.visibility.set(visibility)
            except pm.MayaAttributeError:
                pass  # Skip the element if visibility cannot be set


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    pass

# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------
